@@ -4,12 +4,16 @@
 configuration.py
 
 Created by Thomas Mangin on 2009-08-25.
-Copyright (c) 2009-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
 import unittest
 
-from exabgp.configuration import Configuration
+from exabgp.structure.environment import environment
+env = environment.setup('')
+
+from exabgp.structure.configuration import Configuration
+
 
 class TestConfiguration (unittest.TestCase):
 	def setUp(self):
@@ -308,4 +312,3 @@ neighbor 192.168.127.128 {
 
 if __name__ == '__main__':
 	unittest.main()
-
