@@ -4,11 +4,14 @@
 update.py
 
 Created by Thomas Mangin on 2009-09-06.
-Copyright (c) 2009-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
 import unittest
 
+from exabgp.structure.environment import environment
+env = environment.setup('')
+
 from exabgp.bgp.message.open import *
 
 class TestData (unittest.TestCase):
