@@ -4,10 +4,14 @@
 peer.py
 
 Created by Thomas Mangin on 2009-08-30.
-Copyright (c) 2009-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
 import unittest
+
+from exabgp.structure.environment import environment
+env = environment.setup('')
+
 from exabgp.structure.configuration import Configuration
 from exabgp.structure.supervisor import Supervisor
 
