@@ -3,7 +3,7 @@
 attribute/__init__.py
 
 Created by Thomas Mangin on 2009-11-05.
-Copyright (c) 2009-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
 from struct import pack
@@ -31,3 +31,6 @@ class Attribute (object):
 
 	def __eq__ (self,other):
 		return self.ID == other.ID
+
+	def __ne__ (self,other):
+		return self.ID != other.ID
