@@ -3,7 +3,7 @@
 clusterlist.py
 
 Created by Thomas Mangin on 2012-07-07.
-Copyright (c) 2009-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
 from exabgp.protocol.family import AFI,SAFI
@@ -44,5 +44,4 @@ class ClusterList (Attribute):
 		return '%s' % self.clusters[0]
 
 	def json (self):
-		return '[ %s ]' % ', '.join([str(_) for _ in self.clusters])
-
+		return '[ %s ]' % ', '.join(['"%s"' % str(_) for _ in self.clusters])
