@@ -3,7 +3,7 @@
 debug.py
 
 Created by Thomas Mangin on 2011-03-29.
-Copyright (c) 2011-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2012 Exa Networks. All rights reserved.
 """
 
 import os
@@ -18,7 +18,7 @@ def bug_report (type, value, trace):
 	print
 	print "-"*80
 	print "-- Please provide the information below on :"
-	print "-- http://code.google.com/p/exaproxy/issues/entry"
+	print "-- http://code.google.com/p/exabgp/issues/entry"
 	print "-"*80
 	print
 	print
@@ -48,7 +48,7 @@ def bug_report (type, value, trace):
 	print
 	print "-"*80
 	print "-- Please provide the information above on :"
-	print "-- http://code.google.com/p/exaproxy/issues/entry"
+	print "-- http://code.google.com/p/exabgp/issues/entry"
 	print "-"*80
 	print
 	print
