@@ -3,7 +3,7 @@
 ip.py
 
 Created by Thomas Mangin on 2010-01-15.
-Copyright (c) 2010-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013  Exa Networks. All rights reserved.
 """
 
 import socket
@@ -72,3 +72,9 @@ class Inet (object):
 
 	def __eq__ (self,other):
 		return self.packed == other.packed and self.safi == other.safi
+
+	def __ne__ (self,other):
+		return not self.__eq__(other)
+
+	def __repr__ (self):
+		return "<%s value %s>" % (str(self.__class__).split("'")[1].split('.')[-1],str(self))
