@@ -3,19 +3,21 @@
 configuration.py
 
 Created by Thomas Mangin on 2009-08-25.
-Copyright (c) 2009-2012 Exa Networks. All rights reserved.
+Copyright (c) 2009-2013 Exa Networks. All rights reserved.
 """
 
 import os
 import sys
 import stat
+import time
+
 from pprint import pformat
 from copy import deepcopy
 from struct import pack,unpack
 
-from exabgp.structure.environment import load
+from exabgp.structure.environment import environment
 
-from exabgp.protocol.family import AFI,SAFI
+from exabgp.protocol.family import AFI,SAFI,known_families
 
 from exabgp.bgp.neighbor import Neighbor
 
@@ -40,7 +42,7 @@ from exabgp.bgp.message.update.attribute.med import MED
 from exabgp.bgp.message.update.attribute.localpref import LocalPreference
 from exabgp.bgp.message.update.attribute.atomicaggregate import AtomicAggregate
 from exabgp.bgp.message.update.attribute.aggregator import Aggregator
-from exabgp.bgp.message.update.attribute.communities import Community,cachedCommunity,Communities,ECommunity,ECommunities,to_ExtendedCommunity,to_FlowTrafficRate,to_RouteTargetCommunity_00,to_RouteTargetCommunity_01
+from exabgp.bgp.message.update.attribute.communities import Community,cachedCommunity,Communities,ECommunity,ECommunities,to_ExtendedCommunity,to_FlowTrafficRate,to_FlowRedirectASN,to_FlowRedirectIP
 from exabgp.bgp.message.update.attribute.originatorid import OriginatorID
 from exabgp.bgp.message.update.attribute.clusterlist import ClusterList
 
@@ -64,11 +66,35 @@ class Withdrawn (object):
 	ID = AttributeID.INTERNAL_WITHDRAW
 	MULTIPLE = False
 
+
+def convert_length (data):
+	number = int(data)
+	if number > 65535:
+		raise ValueError(Configuration._str_bad_length)
+	return number
+
+def convert_port (data):
+	number = int(data)
+	if number < 0 or number > 65535:
+		raise ValueError(Configuration._str_bad_port)
+	return number
+
+def convert_dscp (data):
+	number = int(data)
+	if number < 0 or number > 65535:
+		raise ValueError(Configuration._str_bad_dscp)
+	return number
+
+
 class Configuration (object):
 	TTL_SECURITY = 255
 
 #	'  hold-time 180;\n' \
 
+	_str_bad_length = "cloudflare already found that invalid max-packet length for for you .."
+	_str_bad_flow = "you tried to filter a flow using an invalid port for a component .."
+	_str_bad_dscp = "you tried to filter a flow using an invalid dscp for a component .."
+
 	_str_route_error = \
 	'community, extended-communities and as-path can take a single community as parameter.\n' \
 	'only next-hop is mandatory\n' \
@@ -162,7 +188,8 @@ class Configuration (object):
 	'        }\n'
 
 	def __init__ (self,fname,text=False):
-		self.debug = load().debug.configuration
+		self.debug = environment.settings().debug.configuration
+		self.api_encoder = environment.settings().api.encoder
 
 		self.logger = Logger()
 		self._text = text
@@ -220,18 +247,18 @@ class Configuration (object):
 
 		self.neighbor = self._neighbor
 
-		if load().debug.route:
-			self.decode(load().debug.route)
+		if environment.settings().debug.route:
+			self.decode(environment.settings().debug.route)
 			sys.exit(0)
 
-		if load().debug.selfcheck:
+		if environment.settings().debug.selfcheck:
 			self.selfcheck()
 			sys.exit(0)
 
 		return True
 
-	def parse_single_route (self,command):
-		tokens = command.split(' ')[1:]
+	def parse_api_route (self,command):
+		tokens = self._cleaned(command).split(' ')[1:]
 		if len(tokens) < 4:
 			return False
 		if tokens[0] != 'route':
@@ -239,16 +266,16 @@ class Configuration (object):
 		scope = [{}]
 		if not self._single_static_route(scope,tokens[1:]):
 			return None
-		return scope[0]['routes'][0]
+		return scope[0]['routes']
 
-	def parse_single_flow (self,command):
-		self._tokens = self._tokenise(' '.join(command.split(' ')[2:]).split('\\n'))
+	def parse_api_flow (self,command):
+		self._tokens = self._tokenise(' '.join(self._cleaned(command).split(' ')[2:]).split('\\n'))
 		scope = [{}]
 		if not self._dispatch(scope,'flow',['route',],[]):
 			return None
 		if not self._check_flow_route(scope):
 			return None
-		return scope[0]['routes'][0]
+		return scope[0]['routes']
 
 	def add_route_all_peers (self,route):
 		for neighbor in self.neighbor:
@@ -263,12 +290,15 @@ class Configuration (object):
 
 	# Tokenisation
 
+	def _cleaned (self,line):
+		return line.strip().replace('\t',' ').replace(']',' ]').replace('[','[ ').replace(')',' )').replace('(','( ').lower()
+
 	def _tokenise (self,text):
 		r = []
 		config = ''
 		for line in text:
 			self.logger.configuration('loading | %s' % line.rstrip())
-			replaced = line.strip().replace('\t',' ').replace(']',' ]').replace('[','[ ').replace(')',' )').replace('(','( ').lower()
+			replaced = self._cleaned(line)
 			config += line
 			if not replaced:
 				continue
@@ -471,8 +501,22 @@ class Configuration (object):
 
 		elif name == 'process':
 			if command == 'run': return self._set_process_run(scope,'process-run',tokens[1:])
-			if command == 'parse-routes': return self._set_process_parse_routes(scope,'parse-routes',tokens[1:])
-			if command == 'peer-updates': return self._set_process_peer_updates(scope,'peer-updates',tokens[1:])
+			# legacy ...
+			if command == 'parse-routes':
+				self._set_process_command(scope,'neighbor-changes',tokens[1:])
+				self._set_process_command(scope,'receive-routes',tokens[1:])
+				return True
+			# legacy ...
+			if command == 'peer-updates':
+				self._set_process_command(scope,'neighbor-changes',tokens[1:])
+				self._set_process_command(scope,'receive-routes',tokens[1:])
+				return True
+			# new interface
+			if command == 'encoder': return self._set_process_encoder(scope,'encoder',tokens[1:])
+			if command == 'receive-packets': return self._set_process_command(scope,'receive-packets',tokens[1:])
+			if command == 'send-packets': return self._set_process_command(scope,'send-packets',tokens[1:])
+			if command == 'receive-routes': return self._set_process_command(scope,'receive-routes',tokens[1:])
+			if command == 'neighbor-changes': return self._set_process_command(scope,'neighbor-changes',tokens[1:])
 
 		elif name == 'static':
 			if command == 'route': return self._single_static_route(scope,tokens[1:])
@@ -482,29 +526,40 @@ class Configuration (object):
 	# Programs used to control exabgp
 
 	def _multi_process (self,scope,tokens):
-		if len(tokens) != 1:
+		while True:
+			r = self._dispatch(scope,'process',[],['run','encoder','receive-packets','send-packets','receive-routes','neighbor-changes',  'peer-updates','parse-routes'])
+			if r is False: return False
+			if r is None: break
+
+		name = tokens[0] if len(tokens) >= 1 else 'conf-only-%s' % str(time.time())[-6:]
+		self.process.setdefault(name,{})['neighbor'] = scope[-1]['peer-address'] if 'peer-address' in scope[-1] else '*'
+
+		run = scope[-1].pop('process-run','')
+		if run:
+			if len(tokens) != 1:
+				self._error = self._str_process_error
+				if self.debug: raise
+				return False
+			self.process[name]['encoder'] = scope[-1].get('encoder','') or self.api_encoder
+			self.process[name]['run'] = run
+			return True
+		elif len(tokens):
 			self._error = self._str_process_error
 			if self.debug: raise
 			return False
-		while True:
-			r = self._dispatch(scope,'process',[],['run','parse-routes','peer-updates'])
-			if r is False: return False
-			if r is None: break
-		self.process.setdefault(tokens[0],{})['run'] = scope[-1].pop('process-run')
-		self.process[tokens[0]]['receive-routes'] = scope[-1].get('parse-routes',False)
-		if 'peer-address' in scope[-1]:
-			self.process[tokens[0]]['neighbor'] = scope[-1]['peer-address']
-		else:
-			self.process[tokens[0]]['neighbor'] = '*'
-		return True
 
-	def _set_process_parse_routes (self,scope,command,value):
+	def _set_process_command (self,scope,command,value):
 		scope[-1][command] = True
 		return True
 
-	def _set_process_peer_updates (self,scope,command,value):
-		scope[-1][command] = True
-		return True
+	def _set_process_encoder (self,scope,command,value):
+		if value and value[0] in ('text','json'):
+			scope[-1][command] = value[0]
+			return True
+
+		self._error = self._str_process_error
+		if self.debug: raise
+		return False
 
 	def _set_process_run (self,scope,command,value):
 		line = ' '.join(value).strip()
@@ -526,7 +581,7 @@ class Configuration (object):
 				path = [os.environ.get('ETC','etc'),] + parts[2:]
 				prg = os.path.join(*path)
 			else:
-				prg = os.path.join(os.getcwd(),prg)
+				prg = os.path.abspath(os.path.join(os.path.dirname(self._fname),prg))
 		if not os.path.exists(prg):
 			self._error = 'can not locate the the program "%s"' % prg
 			if self.debug: raise
@@ -629,18 +684,6 @@ class Configuration (object):
 		self._family = True
 		return True
 
-	def _all_families (self):
-		# it can not be a generator
-		families = []
-		families.append((AFI(AFI.ipv4),SAFI(SAFI.unicast)))
-		families.append((AFI(AFI.ipv4),SAFI(SAFI.multicast)))
-		families.append((AFI(AFI.ipv4),SAFI(SAFI.nlri_mpls)))
-		families.append((AFI(AFI.ipv4),SAFI(SAFI.mpls_vpn)))
-		families.append((AFI(AFI.ipv4),SAFI(SAFI.flow_ipv4)))
-		families.append((AFI(AFI.ipv6),SAFI(SAFI.unicast)))
-		families.append((AFI(AFI.ipv6),SAFI(SAFI.mpls_vpn)))
-		return families
-
 	# capacity
 
 	def _multi_capability (self,scope,tokens):
@@ -785,15 +828,18 @@ class Configuration (object):
 			v = local_scope.get('hold-time','')
 			if v: neighbor.hold_time = v
 
-			neighbor.parse_routes = local_scope.get('parse-routes',False)
-			neighbor.peer_updates = local_scope.get('peer-updates',False)
 			v = local_scope.get('routes',[])
 			for route in v:
 				# This add the family to neighbor.families()
 				neighbor.add_route(route)
 
-		# drop the neighbor
-		local_scope = scope.pop(-1)
+		for local_scope in (scope[0],scope[-1]):
+			neighbor.api.receive_packets |= local_scope.get('receive-packets',False)
+			neighbor.api.send_packets |= local_scope.get('send-packets',False)
+			neighbor.api.receive_routes |= local_scope.get('receive-routes',False)
+			neighbor.api.neighbor_changes |= local_scope.get('neighbor-changes',False)
+
+		local_scope = scope[-1]
 		neighbor.description = local_scope.get('description','')
 
 		neighbor.md5 = local_scope.get('md5',None)
@@ -829,7 +875,7 @@ class Configuration (object):
 			# announce what is needed, and no more, no need to have lots of TCP session doing nothing
 			families = neighbor.families()
 		elif openfamilies in ('all','everything'):
-			families = self._all_families()
+			families = known_families()
 		# only announce what you have as routes
 		elif openfamilies == 'minimal':
 			families = neighbor.families()
@@ -862,6 +908,11 @@ class Configuration (object):
 				self._neighbor[m_neighbor.name()] = m_neighbor
 		else:
 			self._neighbor[neighbor.name()] = neighbor
+
+		for line in str(neighbor).split('\n'):
+			self.logger.configuration(line)
+		self.logger.configuration("\n")
+
 		return True
 
 
@@ -1208,7 +1259,14 @@ class Configuration (object):
 
 	def _route_next_hop (self,scope,tokens):
 		try:
-			scope[-1]['routes'][-1].attributes.add(cachedNextHop(*inet(tokens.pop(0))))
+			# next-hop self is unsupported
+			ip = tokens.pop(0)
+			if ip.lower() == 'self':
+				la = scope[-1]['local-address']
+				nh = la.afi,la.safi,la.pack()
+			else:
+				nh = inet(ip)
+			scope[-1]['routes'][-1].attributes.add(cachedNextHop(*nh))
 			return True
 		except:
 			self._error = self._str_route_error
@@ -1710,10 +1768,7 @@ class Configuration (object):
 				while test:
 					operator,_ = self._operator(test)
 					value,test = self._value(_)
-					try:
-						number = int(value)
-					except ValueError:
-						number = converter(value)
+					number = converter(value)
 					scope[-1]['routes'][-1].add_or(klass(AND|operator,number))
 					if test:
 						if test[0] == '&':
@@ -1724,8 +1779,8 @@ class Configuration (object):
 						else:
 							raise ValueError("Unknown binary operator %s" % test[0])
 			return True
-		except ValueError:
-			self._error = self._str_route_error
+		except ValueError,e:
+			self._error = self._str_route_error + str(e)
 			if self.debug: raise
 			return False
 
@@ -1767,16 +1822,16 @@ class Configuration (object):
 		return self._flow_generic_list(scope,tokens,converter,klass)
 
 	def _flow_route_anyport (self,scope,tokens):
-		return self._flow_generic_condition(scope,tokens,int,AnyPort)
+		return self._flow_generic_condition(scope,tokens,convert_port,AnyPort)
 
 	def _flow_route_source_port (self,scope,tokens):
-		return self._flow_generic_condition(scope,tokens,int,SourcePort)
+		return self._flow_generic_condition(scope,tokens,convert_port,SourcePort)
 
 	def _flow_route_destination_port (self,scope,tokens):
-		return self._flow_generic_condition(scope,tokens,int,DestinationPort)
+		return self._flow_generic_condition(scope,tokens,convert_port,DestinationPort)
 
 	def _flow_route_packet_length (self,scope,tokens):
-		return self._flow_generic_condition(scope,tokens,int,PacketLength)
+		return self._flow_generic_condition(scope,tokens,convert_length,PacketLength)
 
 	def _flow_route_tcp_flags (self,scope,tokens):
 		return self._flow_generic_list(scope,tokens,NamedTCPFlags,TCPFlag)
@@ -1794,7 +1849,7 @@ class Configuration (object):
 		return self._flow_generic_list(scope,tokens,NamedFragment,Fragment)
 
 	def _flow_route_dscp (self,scope,tokens):
-		return self._flow_generic_condition(scope,tokens,int,DSCP)
+		return self._flow_generic_condition(scope,tokens,convert_dscp,DSCP)
 
 	def _flow_route_discard (self,scope,tokens):
 		# README: We are setting the ASN as zero as that what Juniper (and Arbor) did when we created a local flow route
@@ -1829,22 +1884,24 @@ class Configuration (object):
 			if prefix.count('.'):
 				ip = prefix.split('.')
 				if len(ip) != 4:
-					raise ValueError('')
+					raise ValueError('invalid IP %s' % prefix)
 				ipn = 0
 				while ip:
 					ipn <<= 8
 					ipn += int(ip.pop(0))
 				number = int(suffix)
-				scope[-1]['routes'][-1].add_action(to_RouteTargetCommunity_01(ipn,number))
+				if number >= pow(2,16):
+					raise ValueError('number is too large, max 16 bits %s' % number)
+				scope[-1]['routes'][-1].add_action(to_FlowRedirectIP(ipn,number))
 				return True
 			else:
 				asn = int(prefix)
 				route_target = int(suffix)
 				if asn >= pow(2,16):
-					asn = asn & 0xFFFF
+					raise ValueError('asn is a 32 bits number, it can only be 16 bit %s' % route_target)
 				if route_target >= pow(2,32):
 					raise ValueError('route target is a 32 bits number, value too large %s' % route_target)
-				scope[-1]['routes'][-1].add_action(to_RouteTargetCommunity_00(asn,route_target))
+				scope[-1]['routes'][-1].add_action(to_FlowRedirectASN(asn,route_target))
 				return True
 		except ValueError:
 			self._error = self._str_route_error
@@ -1857,7 +1914,7 @@ class Configuration (object):
 		from exabgp.bgp.message.update import Update
 		from exabgp.bgp.message.open import Open
 		from exabgp.bgp.message.open.capability import Capabilities
-		from exabgp.bgp.message.open.capability.negociated import Negociated
+		from exabgp.bgp.message.open.capability.negotiated import Negotiated
 		from exabgp.bgp.message.open.capability.id import CapabilityID
 
 		self.logger.info('\ndecoding routes in configuration','parser')
@@ -1871,7 +1928,7 @@ class Configuration (object):
 		n = self.neighbor[self.neighbor.keys()[0]]
 
 		path = {}
-		for f in self._all_families():
+		for f in known_families():
 			if n.add_path:
 				path[f] = n.add_path
 
@@ -1881,14 +1938,14 @@ class Configuration (object):
 
 		o1 = Open().new(4,n.local_as,str(n.local_address),capa,180)
 		o2 = Open().new(4,n.peer_as,str(n.peer_address),capa,180)
-		negociated = Negociated()
-		negociated.sent(o1)
-		negociated.received(o2)
+		negotiated = Negotiated()
+		negotiated.sent(o1)
+		negotiated.received(o2)
 		#grouped = False
 
 		injected = ''.join(chr(int(_,16)) for _ in (route[i*2:(i*2)+2] for i in range(len(route)/2)))
 		# This does not take the BGP header - let's assume we will not break that :)
-		update = Update().factory(negociated,injected)
+		update = Update().factory(negotiated,injected)
 		self.logger.info('','parser')
 		for route in update.routes:
 			self.logger.info('decoded route %s' % route.extensive(),'parser')
@@ -1907,7 +1964,7 @@ class Configuration (object):
 		from exabgp.bgp.message.update import Update
 		from exabgp.bgp.message.open import Open
 		from exabgp.bgp.message.open.capability import Capabilities
-		from exabgp.bgp.message.open.capability.negociated import Negociated
+		from exabgp.bgp.message.open.capability.negotiated import Negotiated
 		from exabgp.bgp.message.open.capability.id import CapabilityID
 
 		self.logger.info('\ndecoding routes in configuration','parser')
@@ -1915,7 +1972,7 @@ class Configuration (object):
 		n = self.neighbor[self.neighbor.keys()[0]]
 
 		path = {}
-		for f in self._all_families():
+		for f in known_families():
 			if n.add_path:
 				path[f] = n.add_path
 
@@ -1925,9 +1982,9 @@ class Configuration (object):
 
 		o1 = Open().new(4,n.local_as,str(n.local_address),capa,180)
 		o2 = Open().new(4,n.peer_as,str(n.peer_address),capa,180)
-		negociated = Negociated()
-		negociated.sent(o1)
-		negociated.received(o2)
+		negotiated = Negotiated()
+		negotiated.sent(o1)
+		negotiated.received(o2)
 		#grouped = False
 
 		for nei in self.neighbor.keys():
@@ -1937,12 +1994,12 @@ class Configuration (object):
 				for route in self.neighbor[nei]._routes[family]:
 					str1 = route.extensive()
 					update = Update().new([route])
-					packed = update.announce(negociated)
+					packed = update.announce(negotiated)
 					self.logger.info('parsed route requires %d updates' % len(packed),'parser')
 					for pack in packed:
 						self.logger.info('update size is %d' % len(pack),'parser')
 						# This does not take the BGP header - let's assume we will not break that :)
-						update = Update().factory(negociated,pack[19:])
+						update = Update().factory(negotiated,pack[19:])
 						self.logger.info('','parser')
 						for route in update.routes:
 							str2 = route.extensive()
